﻿package com.mm.android.audiorecord;

public class ADPCMEncode extends AudioEncode
{
	private class IMA_HANDLE {
	    short prev_sample;
	    short step_index;
	};

	/* step_table[] and index_table[] are from the ADPCM reference source */
	/* This is the index table: */
	int[] index_adjust = {
	    -1, -1, -1, -1, 2, 4, 6, 8,
	    -1, -1, -1, -1, 2, 4, 6, 8
	};

	int[] step_table =
	{ 7, 8, 9, 10, 11, 12, 13, 14,
	    16, 17, 19, 21, 23, 25, 28, 31,
	    34, 37, 41, 45, 50, 55, 60, 66,
	    73, 80, 88, 97, 107, 118, 130, 143,
	    157, 173, 190, 209, 230, 253, 279, 307,
	    337, 371, 408, 449, 494, 544, 598, 658,
	    724, 796, 876, 963, 1060, 1166, 1282, 1411,
	    1552, 1707, 1878, 2066, 2272, 2499, 2749, 3024,
	    3327, 3660, 4026, 4428, 4871, 5358, 5894, 6484,
	    7132, 7845, 8630, 9493, 10442, 11487, 12635, 13899,
	    15289, 16818, 18500, 20350, 22385, 24623, 27086, 29794, 32767 };
	
	//ENCODERS
	char encode_one_sample(IMA_HANDLE p, short sample)
	{
	    int code; /*adpcm output value */
	    int diff; /*different between sample and the predicted sample*/
	    int tempstep;
	    int step; /*quantizer step size*/
	    int predsample; /*ouput of adpcm predictor*/
	    int diffq; /*quantized predicted difference*/
	    int index; /*index of step size table*/
	    
	    predsample = p.prev_sample;
	    index = p.step_index;
	    step = step_table[index];
	    
	    diff = sample - predsample;
	    if (diff >= 0)
	        code = 0;
	    else
	    {
	        code = 8;
	        diff = -diff;
	    }
	    
	    /*quantize the difference into the 4-bit adpcm code unsing the	quantizer step size*/
	    /*inverse quantize the adpcm code into a predicted difference using the quantizer step size*/
	    tempstep = step;
	    if (diff >= tempstep)
	    {
	        code |= 4;
	        diff -= tempstep;
	    }
	    tempstep >>= 1;
	    if (diff >= tempstep)
	    {
	        code |= 2;
	        diff -= tempstep;
	    }
	    tempstep >>= 1;
	    if (diff >= tempstep)
	        code |= 1;
	    
	    diffq = step >> 3;
	    if ((code & 4) != 0)
	        diffq += step;
	    if ((code & 2) != 0)
	        diffq += step >> 1;
	    if ((code & 1) != 0)
	        diffq += step >> 2;
	    
	    /*fixed predictor computes new predicted sample by adding the
	     old predicted sample to predicted difference*/
	    if((code & 8) != 0)
	        predsample -= diffq;
	    else
	        predsample += diffq;
	    if( predsample > 32767)
	        predsample = 32767;
	    else if (predsample < -32768)
	        predsample = -32768;
	    /* Find new quantizer stepsize index by adding the old index
	     to a table lookup using the ADPCM code
	     */
	    index += index_adjust[code];
	    /* Check for overflow of the new quantizer step size index
	     */
	    if( index < 0 )
	        index = 0;
	    if( index > 88 )
	        index = 88;
	    /* Save the predicted sample and quantizer step size index for
	     next iteration
	     */
	    p.prev_sample = (short)predsample;
	    p.step_index = (short)index;
	    
	    return (char)(code & 0x0f);
	}
	
	@Override
	public int encode(byte[] pSrcBuf, int nSrcOffset, int nSrcLen, byte[] pDestBuf, int nDestLen)
	{
		int src_index = 0;
	    int dest_index = 0;
	    char result;
	    
	    IMA_HANDLE handle = new IMA_HANDLE();
	    handle.prev_sample = 0x0c;
	    handle.step_index = 0x0c;
	    pDestBuf[1] = (byte)(handle.prev_sample & 0xff);
	    pDestBuf[0] = (byte)(handle.prev_sample >> 8);
	    pDestBuf[3] = (byte)(handle.step_index & 0xff);
	    pDestBuf[2] = (byte)(handle.step_index >> 8);
	    
	    dest_index += 4;
	    
	    while (src_index < nSrcLen / 2)
	    {
	    	short sample = (short)((pSrcBuf[src_index++] >> 8) | pSrcBuf[src_index++]);
	        result  = (char)(encode_one_sample(handle, sample) & 0x0f);
	    	sample = (short)((pSrcBuf[src_index++] >> 8) | pSrcBuf[src_index++]);
	        result |= (encode_one_sample(handle, sample) & 0x0f) << 4;
	        pDestBuf[dest_index++] = (byte)result;
	    }
	    
	    return dest_index;
	}
}
