﻿package com.mm.android.audiorecord;

import java.nio.ByteBuffer;

import com.mm.android.avplaysdk.AVPlaySDK.BIT_PERSAMPLE;
import com.mm.android.avplaysdk.AVPlaySDK.CHANNEL;
import com.mm.android.avplaysdk.AVPlaySDK.SAMPLE_PERSEC;
import com.mm.android.avplaysdk.nativeplayer.NativePlayer;
import com.mm.android.avplaysdk.render.AudioRender;

import android.annotation.SuppressLint;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.media.audiofx.AcousticEchoCanceler;
import android.media.audiofx.AutomaticGainControl;
import android.media.audiofx.NoiseSuppressor;
import android.os.Build;
import android.util.Log;

@SuppressLint("NewApi")
public class SoundRecord implements Runnable
{
	public static final int ERR_NOERROR		  =  0;		// 没有错误
	public static final int ERR_NOT_SURRPORT  = -1;		// 不支持的类型
	public static final int ERR_BUF_TOO_SMALL = -2;		// buffer太小
	public static final int ERR_ORDER		  = -3;		// 调用顺序不对
	
	private static final int MAX_AUDIO_RECORD_LEN 	= 1024;
	private int 			 RECORD_BUFFER_LEN 	  	= 2048;
	
	private AudioRecord 		 m_audioRecord   		= null;;
	private IAudioRecordListener m_audioListener 		= null;
	private byte[]				 m_audioBuffer	 		= null;
	private ByteBuffer			 m_audioBufferStack 	= null;
	private int					 m_BitsPerSample;							//采样深度
	private int					 m_SamplesPerSecond;						//采样率
	private ByteBuffer			 m_audioPlayByffer 		= null;  			//音频播放时的备份数据用于进行回音消除
	private byte[]			 	 m_playByfferLock  		= new byte[1];  	//音频播放时的备份数据用于进行回音消除
	private int					 m_BitsPerSampleRef;						//参考采样深度
	private int					 m_SamplesPerSecondRef  = -1;				//参考采样率
	private Thread				 m_threadRecord  		= null;
	private boolean				 m_bStop		 		= false;
	private int					 m_nMinBufSize	 		= 0;
	private long				 m_echo_cancel   		= -1;				//回音消除句柄
	private AudioRender			 mAudioRender    		= null;
	private byte[]				 mSignalBuffer			= null;
	
	private SFAudioTrack mSFAudioTrack;										//AudioTrack
	private AcousticEchoCanceler aec = null;										//AEC对象
	private AutomaticGainControl agc = null;										//AGC对象
	private NoiseSuppressor ns = null;												//NS对象
	
	public SoundRecord(IAudioRecordListener audioListener)
	{
		m_audioListener = audioListener;
	}
	
	// 开始录音
	public int startAudioRecord(SAMPLE_PERSEC eSamplePerSec, BIT_PERSAMPLE eBitPerSample, CHANNEL eChannel, int nBufLen)
	{
		m_bStop = false;
		
		int nSampleRate  = getSampleRate(eSamplePerSec);
		int nchannel 	 = getChannel(eChannel);
		int nAudioFormat = getAudioFormat(eBitPerSample);
		if ((-1 == nSampleRate) || (-1 == nchannel) || (-1 == nAudioFormat))
		{
			return ERR_NOT_SURRPORT;
		}

		// 获得最小的buffer大小
		m_nMinBufSize = AudioRecord.getMinBufferSize(nSampleRate, nchannel,
				nAudioFormat);
		Log.i("SoundRecord", "===AudioRecord.MinBufferSize="+m_nMinBufSize);
		m_audioBuffer = new byte[m_nMinBufSize];
		int nRetValue = ERR_NOERROR;
		try 
		{
			//需要根据系统SDK版本创建AudioRecord。Virsion11以上使用系统自带回音消除模块，以下使用回音消除库
			if (Build.VERSION.SDK_INT >= 11)
			{
				m_audioRecord = new AudioRecord(MediaRecorder.AudioSource.MIC,
						nSampleRate, nchannel, nAudioFormat, m_nMinBufSize);
				if (Build.VERSION.SDK_INT >= 16) 
				{
					if (AcousticEchoCanceler.isAvailable()) 
					{
						aec = AcousticEchoCanceler.create(m_audioRecord
								.getAudioSessionId());
						if (aec != null)
							aec.setEnabled(true);
					}
					if (AutomaticGainControl.isAvailable()) 
					{
						agc = AutomaticGainControl.create(m_audioRecord
								.getAudioSessionId());
						if (agc != null)
							agc.setEnabled(false);
					}
					if (NoiseSuppressor.isAvailable()) 
					{
						ns = NoiseSuppressor.create(m_audioRecord
								.getAudioSessionId());
						if (ns != null)
							ns.setEnabled(true);
					}
				}
			}
			else
			{
				m_audioRecord = new AudioRecord(MediaRecorder.AudioSource.MIC,
						nSampleRate, nchannel, nAudioFormat, m_nMinBufSize);
			}
			if (AudioRecord.STATE_INITIALIZED == m_audioRecord.getState())
			{
				m_audioRecord.startRecording();
				m_threadRecord = new Thread(this);
				m_threadRecord.start();
				nRetValue = ERR_NOERROR;
			} 
			else
			{
				Log.e("SoundRecord", "AudioRecord is not inited. state="+m_audioRecord.getState());
				m_audioRecord.release();
				m_audioRecord = null;
				nRetValue = ERR_NOT_SURRPORT;
			}
		} 
		catch (IllegalArgumentException exception) 
		{
			exception.printStackTrace();
		} 
		catch (IllegalStateException exception) 
		{
			exception.printStackTrace();
		}

		m_echo_cancel = NativePlayer.echoInit(5000);

		m_audioPlayByffer = ByteBuffer.allocate(RECORD_BUFFER_LEN);
		
		return nRetValue;
	}
	
	// 停止录音
	public int stopAudioRecord()
	{
		if (null == m_audioRecord)
		{
			return ERR_ORDER;
		}
		
		m_bStop = true;
		
		try
		{
			m_threadRecord.join();
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
		
		//停止AudioTrack，释放音频控制对象
		if(mSFAudioTrack!=null)
		{
			mSFAudioTrack.stop();
		}
        if(Build.VERSION.SDK_INT >= 16)
        {
        	if(aec!=null)
        	{
        		aec.release();
        	}
        	if(agc!=null)
        	{
        		agc.release();
        	}
        	if(ns!=null)
        	{
        		ns.release();
        	}
        }
		
		m_audioRecord.stop();
		m_audioRecord.release();
		m_audioRecord = null;
		m_audioBuffer = null;
		
		if(m_echo_cancel != -1)
		{
			NativePlayer.echoDeInit(m_echo_cancel);
			m_echo_cancel = -1;
		}
		
		if(m_audioPlayByffer != null)
		{
			synchronized (m_playByfferLock)
			{
				m_audioPlayByffer.clear();
				m_audioPlayByffer = null;
			}
			
		}
		
		if(m_audioBufferStack != null)
		{
			synchronized (m_audioBufferStack)
			{
				m_audioBufferStack.clear();
				m_audioBufferStack = null;
			}
			
		}
//		closeFile();
		return ERR_NOERROR;
	}
	
	public void backAudioData(byte[] audioBuf, int offset, int length)
	{
		
		if(Build.VERSION.SDK_INT >= 11)
		{
			if(mSFAudioTrack!=null)
			{
				mSFAudioTrack.write(audioBuf, offset, length);
			}
		}
		else
		{
			putEnchData(audioBuf, offset, length);
		}
	}
	
	public void backAudioParam(int nChannel, int nBitsPerSample, int nSamplesPerSecond,AudioRender audioRender)
	{
		//****  audiaTrack start
		mSFAudioTrack = new SFAudioTrack();
		int channel = AudioFormat.CHANNEL_OUT_DEFAULT;
		int nAudioFormat = AudioFormat.ENCODING_DEFAULT;
		
		if (nChannel == 1)
		{
			channel = AudioFormat.CHANNEL_OUT_MONO;
		}
		else if (nChannel == 2)
		{
			channel = AudioFormat.CHANNEL_OUT_STEREO;
		}
		else
		{
			channel = AudioFormat.CHANNEL_OUT_DEFAULT;
		}
		
		if (nBitsPerSample == 8)
		{
			nAudioFormat = AudioFormat.ENCODING_PCM_8BIT;
		}
		else if (nBitsPerSample == 16)
		{
			nAudioFormat = AudioFormat.ENCODING_PCM_16BIT;
		}
		else
		{
			nAudioFormat = AudioFormat.ENCODING_DEFAULT;
		}
			
		if(Build.VERSION.SDK_INT>=16)
		{
			mSFAudioTrack.init(nSamplesPerSecond, channel, nAudioFormat, m_audioRecord.getAudioSessionId());
		}
		else
		{
			mSFAudioTrack.init(nSamplesPerSecond, channel, nAudioFormat);
		}
		mSFAudioTrack.play();
		//******audiaTrack end
		
		m_BitsPerSampleRef = nBitsPerSample;
		m_SamplesPerSecondRef = nSamplesPerSecond;
		mSignalBuffer = NativePlayer.echoGetSignalBuffer();
		if (m_SamplesPerSecondRef == 16000)
		{
			byte[] tmp = new byte[mSignalBuffer.length * 2];
			for (int i = 0; i < mSignalBuffer.length; i = i + 2) 
			{
				System.arraycopy(mSignalBuffer, i, tmp, i * 2, 2);
				System.arraycopy(mSignalBuffer, i, tmp, i * 2 + 2, 2);
			}
			mSignalBuffer = tmp;
		}
		mAudioRender = audioRender;
	}
	
	private int getSampleRate(SAMPLE_PERSEC eSamplePerSec)
	{
		int nSampleRate = -1;
		
		switch (eSamplePerSec)
		{
			case SAMPLE_PERSEC_8000:
				nSampleRate = 8000;
				m_BitsPerSample = 8000;
				break;
				
			case SAMPLE_PERSEC_16000:
				nSampleRate = 16000;
				m_BitsPerSample = 16000;
				break;
				
			case SAMPLE_PERSEC_48000:
				nSampleRate = 48000;
				m_BitsPerSample = 48000;
				break;
				
			default:
				nSampleRate = 0;
				break;	
		}
		
		return nSampleRate;
	}
	
	private int getChannel(CHANNEL eChannel)
	{
		int nChannel = -1;
		
		switch (eChannel)
		{
			case CHANNEL_MONO:
				nChannel = AudioFormat.CHANNEL_IN_MONO;
				break;
			case CHANNEL_STEREO:
				nChannel = AudioFormat.CHANNEL_IN_STEREO;
				break;
			default:
				nChannel = -1;
				break;
		}
		
		return nChannel;
	}
	
	private int getAudioFormat(BIT_PERSAMPLE eBitPerSample)
	{
		int nAudioFormat = -1;
		
		switch (eBitPerSample)
		{
			case BIT_PERSAMPLE_8:
				nAudioFormat = AudioFormat.ENCODING_PCM_8BIT;
				m_SamplesPerSecond = 8;
				break;
				
			case BIT_PERSAMPLE_16:
				nAudioFormat = AudioFormat.ENCODING_PCM_16BIT;
				m_SamplesPerSecond = 16;
				break;
				
			default:
				nAudioFormat = -1;
				break;
		}
		
		return nAudioFormat;
	}

	@Override
	public void run()
	{
		int nReadLen = 0;
		while (!m_bStop)
		{
			nReadLen = m_audioRecord.read(m_audioBuffer, 0, MAX_AUDIO_RECORD_LEN);
			if ((AudioRecord.ERROR_INVALID_OPERATION == nReadLen) ||
				(AudioRecord.ERROR_BAD_VALUE == nReadLen) ||
				(AudioRecord.ERROR == nReadLen))
			{
				//Log.e("AVPlaySDK", "AudioRecorder read error: " + nReadLen);
				try
				{
					Thread.sleep(5);
				} 
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
				
				continue;
			}
			else if (nReadLen > 0)
			{
				m_audioListener.onAudioRecord(m_audioBuffer, 0, nReadLen);
				
//				if(Build.VERSION.SDK_INT>=11)
//				{
//					//大于Version11 直接回调
//					m_audioListener.onAudioRecord(m_audioBuffer, 0, nReadLen);
//				}
//				else
//				{
//					//小于Version11 进行回音消除
//					if (m_audioPlayByffer != null && m_SamplesPerSecondRef != -1&& mAudioRender != null ) 
//					{
//							int enchLength = m_SamplesPerSecondRef/m_BitsPerSample * nReadLen;
//							byte[] ench = getEnchData(enchLength);//获取到参考数据
//							if(ench!=null)
//							{
//								//输出参考“点”数据
//								if (mSignalBuffer != null && mSignalBuffer.length > 0) 
//								{
//									audioTrackDate(mSignalBuffer, 0,mSignalBuffer.length);
//									mSignalBuffer = null;
//								}
//								
//								System.arraycopy(m_audioBuffer, 0, m_audioBuffer, 0, nReadLen);//采集到的正式数据
//								byte[] out = new byte[nReadLen + 256];//输出数据 为什么要+256 fe：（length：1600  回音消除 前3次 1536  但是第4次会把前面 3次遗漏的数据 补充上 变成1792 导致库越界 闪退）
//								int ret = 0 ;
//								ret = NativePlayer.echoCancel(m_echo_cancel, m_audioBuffer, ench,out,m_SamplesPerSecond, m_BitsPerSample,m_BitsPerSampleRef, m_SamplesPerSecondRef);
//								if(ret>0)
//								{
//									//ret>0表示回音消除执行OK
//									ret = Math.min(ret, out.length);
//									m_audioListener.onAudioRecord(out, 0, ret);
//								}
//								else
//								{
//									//失败则发送采集数据
//									m_audioListener.onAudioRecord(m_audioBuffer, 0, nReadLen);
//								}
//								//输出参考音频
//								audioTrackDate(ench, 0, enchLength);
//							}
//							else
//							{
//								m_audioListener.onAudioRecord(m_audioBuffer, 0,nReadLen);
//							}
//					} 
//					else 
//					{
//						m_audioListener.onAudioRecord(m_audioBuffer, 0, nReadLen);
//					}
//				}
			}
		}
	}
	
	/**
	 * 回调回音消除之后的音频数据
	 * @param data
	 * @param offset
	 * @param length
	 */
	private void audioTrackDate(byte[] data, int offset, int length) 
	{
		if(mSFAudioTrack !=null)
		{
			mSFAudioTrack.write(data, offset, length);
		}
	}
	
	/**
	 * 获取参考数据
	 * @param length
	 * @return
	 */
	private byte[] getEnchData(int length)
	{
		byte[] bytes=null;
		if (m_audioPlayByffer != null) 
		{
			synchronized (m_playByfferLock) 
			{
				m_audioPlayByffer.flip();
				if(m_audioPlayByffer.limit()>=length)
				{
					bytes = new byte[length];
					m_audioPlayByffer.get(bytes);
					if(m_audioPlayByffer.remaining()>0)
					{
						byte[] b1  = new byte[m_audioPlayByffer.remaining()];
						m_audioPlayByffer.get(b1);
						m_audioPlayByffer.clear();
						m_audioPlayByffer.put(b1);
					}
				}
				else
				{
					byte[] b1  = new byte[m_audioPlayByffer.limit()];
					m_audioPlayByffer.get(b1);
					m_audioPlayByffer.clear();
					m_audioPlayByffer.put(b1);
				}
			}
		}
		return bytes;
	}
	
	/**
	 * 保存参考数据
	 * @param b
	 * @param offset
	 * @param length
	 */
	private void putEnchData(byte[] b ,int offset,int length)
	{
		if (m_audioPlayByffer != null) 
		{
			synchronized (m_playByfferLock) 
			{
				int remaining = m_audioPlayByffer.remaining();
				if(remaining<length)
				{
					m_audioPlayByffer.flip();
					byte[] others = new byte[m_audioPlayByffer.limit()];
					m_audioPlayByffer.get(others);
					m_audioPlayByffer = ByteBuffer.allocate(others.length+length);
					m_audioPlayByffer.clear();
					m_audioPlayByffer.put(others);
				}
				m_audioPlayByffer.put(b, offset, length);
			}
		}
	}
	
	public int getMinBufferSize(SAMPLE_PERSEC eSamplePerSec, BIT_PERSAMPLE eBitPerSample, CHANNEL eChannel)
	{
		m_nMinBufSize = AudioRecord.getMinBufferSize(getSampleRate(eSamplePerSec), 
													 getChannel(eChannel), 
													 getAudioFormat(eBitPerSample));
		
		return m_nMinBufSize;
	}
}

































