﻿package com.mm.android.audiorecord;

import com.mm.android.avplaysdk.AVPlaySDK.AUDIO_ENC_TYPE;
import com.mm.android.avplaysdk.AVPlaySDK.BIT_PERSAMPLE;
import com.mm.android.avplaysdk.AVPlaySDK.CHANNEL;
import com.mm.android.avplaysdk.AVPlaySDK.SAMPLE_PERSEC;
import com.mm.android.avplaysdk.render.AudioRender;

public class VoiceIntercom implements IAudioRecordListener
{
	private static final int 	   AUDIO_HEAD_LEN 		  = 8;
	
	private IVoiceIntercomListener m_voiceIntercomListner = null;
	private AudioEncode 		   m_audioEncoder 		  = null;
	private SoundRecord			   m_soundRecorder		  = null;
	private byte[]				   m_audioData			  = null;
	private AUDIO_ENC_TYPE	   	   m_eAudioEncType		  = AUDIO_ENC_TYPE.AET_PCM16;
	private	static VoiceIntercom   m_voiceIntercom		  = null;
	private static byte[]		   m_voiceIntercomLock	  = new byte[0];
	private SAMPLE_PERSEC 		   m_audioSamplePerSec    = SAMPLE_PERSEC.SAMPLE_PERSEC_8000;
	
	private VoiceIntercom()
	{
	}
	
	public static VoiceIntercom instance()
	{
		if (null == m_voiceIntercom)
		{
			synchronized (m_voiceIntercomLock)
			{
				if (null == m_voiceIntercom)
				{
					m_voiceIntercom = new VoiceIntercom();
				}
			}
		}
		
		return m_voiceIntercom;
	}
	
	public boolean start(IVoiceIntercomListener voiceIntercomListner,
						 SAMPLE_PERSEC eSamplePerSec, BIT_PERSAMPLE eBitPerSample, 
						 CHANNEL eChannel, AUDIO_ENC_TYPE eAudioEncType)
	{
		m_voiceIntercomListner = voiceIntercomListner;
		
		// 创建音频编码器
		m_audioEncoder = getAudioEncoder(eAudioEncType);
		if (null == m_audioEncoder && AUDIO_ENC_TYPE.AET_PCM16 != eAudioEncType &&
			AUDIO_ENC_TYPE.AET_PCM8 != eAudioEncType)
		{
			return false;
		}
		m_eAudioEncType = eAudioEncType;
		m_audioSamplePerSec = eSamplePerSec;
		
		// 创建音频采集器
		m_soundRecorder = new SoundRecord(this);
		if (null == m_soundRecorder)
		{
			m_audioEncoder = null;
			return false;
		}
		
		// 开始采集音频
		int nMinBufSize = m_soundRecorder.getMinBufferSize(eSamplePerSec, eBitPerSample, eChannel);
		m_audioData 	= new byte[nMinBufSize + AUDIO_HEAD_LEN];
		int nRetValue   = m_soundRecorder.startAudioRecord(eSamplePerSec, eBitPerSample, eChannel, nMinBufSize);
		if (SoundRecord.ERR_NOERROR != nRetValue)
		{			
			m_audioEncoder  = null;
			m_soundRecorder = null;
			
			return false;
		}
		
		return true;
	}
	
	public boolean stop()
	{
		if (null == m_soundRecorder)
		{
			return false;
		}
		
		if(m_soundRecorder.stopAudioRecord() != SoundRecord.ERR_NOERROR)
		{
			return false;
		}
		m_audioEncoder  = null;
		m_soundRecorder = null;
		m_audioData		= null;
				
		return true;
	}

	public boolean isRecord()
	{
		if(m_soundRecorder == null)
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	
	/**
	 * 将音频播放的数据回调到采集模块作为参考
	 * @param audioBuf
	 */
	public void backAudioData(byte[] audioBuf,int nOffset, int nDataLen)
	{
		if(m_soundRecorder != null)
		{
			m_soundRecorder.backAudioData(audioBuf,nOffset,nDataLen);
		}
	}
	
	/**
	 * 回调音频参数
	 * @param nBitsPerSample
	 * @param nSamplesPerSecond
	 * @param audioRender
	 */
	public void backAudioParam(int nChannels, int nBitsPerSample, int nSamplesPerSecond,AudioRender audioRender)
	{
		if(m_soundRecorder != null)
		{
			m_soundRecorder.backAudioParam(nChannels, nBitsPerSample, nSamplesPerSecond,audioRender);
		}
	}
	@Override
	public void onAudioRecord(byte[] audioBuf, int nOffset, int nLength)
	{
		if (null != m_voiceIntercomListner)
		{
			int nDataLen = 0;
			if (null != m_audioEncoder)
			{
				nDataLen = m_audioEncoder.encode(audioBuf, nOffset, nLength, m_audioData, AUDIO_HEAD_LEN);
//				m_voiceIntercomListner.onVoiceData(m_audioData, 0, nDataLen);
			}
			else if (AUDIO_ENC_TYPE.AET_PCM16 == m_eAudioEncType)
			{
				System.arraycopy(audioBuf, nOffset, m_audioData, AUDIO_HEAD_LEN, nLength);
				nDataLen = nLength;
//				m_voiceIntercomListner.onVoiceData(audioBuf, nOffset, nLength);
			}
			else
			{
				// nothing
			}
			
			nDataLen += packetAudioHead(nDataLen);
			
			m_voiceIntercomListner.onVoiceData(m_audioData, nOffset, nDataLen);
		}
	}
	
	private AudioEncode getAudioEncoder(AUDIO_ENC_TYPE eAudioEncType)
	{
		AudioEncode audioEncoder = null;
		
		switch (eAudioEncType)
		{
			case AET_G711A:
				audioEncoder = new G711AEncode();
				break;
				
			case AET_G711U:
				audioEncoder = new G711UEncode();
				break;
				
			default:
				audioEncoder = null;
				break;
		}
		
		return audioEncoder;
	}
	
	private int packetAudioHead(int nDataLen)
	{
		m_audioData[0] = (byte) 0x00;
		m_audioData[1] = (byte) 0x00;
		m_audioData[2] = (byte) 0x01;
		m_audioData[3] = (byte) 0xF0;
		switch (m_eAudioEncType)
		{
			case AET_PCM8:
				m_audioData[4] = (byte) 0x07;
				break;
				
			case AET_PCM16:
				m_audioData[4] = (byte) 0x0C;
				break;
				
			case AET_G711A:
				m_audioData[4] = (byte) 0x0E;
				break;
				
			case AET_G711U:
				m_audioData[4] = (byte) 0x0A;
				break;
				
			default:
				m_audioData[4] = (byte) 0x0C;
				break;
		}
		
		// add support SAMPLE_PERSEC_16000 by 23139
		switch (m_audioSamplePerSec)
		{
			case SAMPLE_PERSEC_8000:
				m_audioData[5] = (byte) 0x02;
				break;
			case SAMPLE_PERSEC_16000:
				m_audioData[5] = (byte) 0x04;
				break;
			case SAMPLE_PERSEC_48000:
				m_audioData[5] = (byte) 0x09;
				break;
			default:
				m_audioData[5] = (byte) 0x02;
				break;
		}
		
		m_audioData[6] = (byte) (nDataLen & 0x0FF);
		m_audioData[7] = (byte) ((nDataLen >> 8) & 0x0FF);
		
		return AUDIO_HEAD_LEN;
	}
}
