﻿package com.mm.android.avplaysdk;

import java.io.File;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import android.util.Log;

import com.mm.android.audiorecord.VoiceIntercom;
import com.mm.android.avplaysdk.nativeplayer.NativePlayer;
import com.mm.android.avplaysdk.param.AVFrameInfo;
import com.mm.android.avplaysdk.render.AudioRender;
import com.mm.android.avplaysdk.render.BasicGLSurfaceView;
import com.mm.android.avplaysdk.render.BasicSurfaceView;
import com.mm.android.avplaysdk.render.IRenderListener;


public abstract class AVPlayer implements IPlayHandle, IRenderListener
{
	enum PLAY_STATUS
	{
		PS_INIT, PS_PLAY, PS_PAUSE, PS_STOP
	}
	
	private static final int MIN_RATE		 = -4;
	private static final int MAX_RATE		 = 4;
	private static final String FILE_DIR_BASE = "/sdcard";
	
	protected int ENGINE_MIN_THRESHOLD 		 = 5000;		// 5S 
	
	protected long 			  m_nativePlayer = 0;
	private   Timer 		  m_timer		 = null;
	private   RenderTimerTask m_task		 = null;
	private   IPlayView		  m_playView	 = null;
	protected IPlayListener	  m_playListener = null;
	private	  PLAY_STATUS	  m_ePlayStatus  = PLAY_STATUS.PS_INIT;
	private	  int			  m_nFrameRate	 = 25;
	private	  int			  m_nWidth		 = 0;
	private	  int			  m_nHeight		 = 0;
	private	  int			  m_nSpeed		 = 0;
//	private	  AudioRender	  m_audioRender	 = null;
	private   Map<Integer, AudioRender>	m_audioRender = new HashMap<Integer, AudioRender>();
	private	  FileRecorder	  m_record		 = null;
	private	  boolean		  m_byFrame		 = false;
	private   boolean		  m_playSound	 = false;
	
	// SDK初始化
	public final static int startup(String strAppPath,String strPlayBackDir){
	
		int nInitRetValue = NativePlayer.startup(strAppPath,strPlayBackDir);
		int nRetValue 	  = AVPlaySDK.AVPLAY_OK;
		if (NativePlayer.INIT_ERR_OK == nInitRetValue)
		{
			nRetValue = AVPlaySDK.AVPLAY_OK;			
		}
		else if (NativePlayer.INIT_ERR_DECODER == nInitRetValue)
		{
			nRetValue = AVPlaySDK.AVPLAY_DECODER_INIT_FAILED;
		}
		else if (NativePlayer.INIT_ERR_PLAYBACK == nInitRetValue)
		{
			nRetValue = AVPlaySDK.AVPLAY_PLAYBACK_INIT_FAILED;
		}
		else
		{
			nRetValue = AVPlaySDK.AVPLAY_ERROR;
		}
		if (!initPlaybackDir(strPlayBackDir))
		{
			nRetValue = AVPlaySDK.AVPLAY_ERROR;
		}
		
		return nRetValue;
	}
	
	private static boolean initPlaybackDir(String strAppPath)
	{
		boolean retValue = true;
		
		String fileDir = String.format(Locale.US,"%s/%s", FILE_DIR_BASE, strAppPath);
		File file = new File(fileDir);
		if (!file.exists())
		{
			retValue = file.mkdirs();
		}
		
		return retValue;
	}

	// SDK退出清理
	public final static boolean cleanup()
	{
		return NativePlayer.cleanup();
	}
	
	class RenderTimerTask extends TimerTask
	{
		@Override
		public void run()
		{
			if (m_ePlayStatus == PLAY_STATUS.PS_PLAY)
			{
				if(!m_byFrame)
				{
					m_playView.render();
				}
			}
		}
	}
	
	public void setPlayOneFrameMode(boolean byFrame)
	{
		m_byFrame = byFrame;
		controlAudio();
		m_playView.setFrameMode(byFrame);
	}
	
	public void playOneFrame()
	{
		if (m_ePlayStatus == PLAY_STATUS.PS_PLAY)
		{
			if(m_byFrame)
			{
				m_playView.render();
			}
			
		}
	}
	
	// 打开
	public boolean open(int nDelayTime, boolean bPureAudio)
	{
		m_nativePlayer = NativePlayer.createPlayer(NativePlayer.PARSE_MODE_STREAM, nDelayTime, bPureAudio);
		if (0 == m_nativePlayer)
		{
			return false;
		}
		
		changePlayStatus(PLAY_STATUS.PS_INIT);
		
		return true;
	}

	
	// 关闭
	public void close()
	{
		m_playSound = false;
		if (0 != m_nativePlayer)
		{
			NativePlayer.destroyPlayer(m_nativePlayer);
			m_nativePlayer = 0;
			changePlayStatus(PLAY_STATUS.PS_INIT);
		}
	}
	
	// 开始播放
	public boolean play(IPlayView playView, IPlayListener playListener)
	{
		if (0 == m_nativePlayer)
		{
			return false;
		}
		
		m_playView 	   = playView;
		m_playListener = playListener;

		NativePlayer.setRenderListener(m_nativePlayer, this);
		
		// 创建显示的定时器
//		m_timer = new Timer();
//		m_task  = new RenderTimerTask();
		
		boolean bRetValue = NativePlayer.play(m_nativePlayer, 0);
		if (bRetValue)
		{
			changePlayStatus(PLAY_STATUS.PS_PLAY);
		}
		
		if (null != m_playView)
		{
			m_playView.setPlayer(this);
			m_playView.start();
			changePlaySpeed(m_nFrameRate);
		}
		
		return bRetValue; 
	}
	
	// 停止播放
	public boolean stop()
	{
		m_playSound = false;
		if (0 == m_nativePlayer)
		{
			return false;
		}
		boolean render = false;
		if (null != m_playView)
		{
			// 关闭前先强制将视频状态置为播放状态，防止关闭时的死锁问题
			changePlayStatus(PLAY_STATUS.PS_PLAY);
			render = m_playView.getEnableRender();
			// 等待显示线程退出
			if(m_byFrame || m_nWidth == 0 || m_nHeight == 0)  //  m_nWidth == 0 || m_nHeight == 0 表示还没有来数据，定时器还没起来，因此需要将显示置为false，否则会死锁
			{
				m_playView.enableRender(false);  //如果是按帧模式下，因为定时器不再刷新View因此这里需要将显示置为false，否则会死锁
			}
			// 直接在关闭的时候将视图置为不可绘画状态，防止死锁
			m_playView.stop();
			
			// 刷新窗口
			if(m_playView instanceof BasicGLSurfaceView)
			{
				((BasicGLSurfaceView) m_playView).requestRender();
			}
			else
			{
				m_playView.render();
			}
			
			if (null != m_task)
			{
				m_task.cancel();
				m_task = null;
			}
			
			if (null != m_timer)
			{
				m_timer.cancel();
				m_timer = null;
			}
			
		
		}
		
		NativePlayer.stop(m_nativePlayer);
		
		changePlayStatus(PLAY_STATUS.PS_STOP);
		if(m_playView != null)
		{
			m_playView.enableRender(render);  
		}
		return true;
	}
	
	/**
	 * 重置播放器
	 * @return
	 */
	public boolean resetPlayer() {
		if (0 == m_nativePlayer)
		{
			return false;
		}
		NativePlayer.resetPlayer(m_nativePlayer);
		return true;
	}
	
	// 输入数据流
	public boolean inputData(byte[] streamData, int nOffset, int nDataLen, boolean bPlay)
	{
		return false;
	}
	
	// 输入数据帧
	public boolean inputFrame(AVFrameInfo hFrameInfo)
	{
		return false;
	}
	
	// 拖动
	public boolean setPlayPos(int nPlayTime)
	{
		return true;
	}
	
	// 暂停与继续
	public boolean pause(boolean bPause)
	{
		PLAY_STATUS ePlayStatus = bPause ? PLAY_STATUS.PS_PAUSE : PLAY_STATUS.PS_PLAY;
		
		changePlayStatus(ePlayStatus);
		
		//enableRender(!bPause);
//		if (null != m_playView)
//		{
//			m_playView.enableRender(!bPause);			
//		}
		
		return true;
	}
	
	// 调整播放速度，nSpeed(-1：慢放1倍，0：正常速度，1：快放1倍)
	public boolean setPlaySpeed(int nMultiSpeed)
	{
		switch (nMultiSpeed)
		{
		case -1:
			m_nSpeed += -2;
			break;
			
		case 0:
			m_nSpeed = 0;
			break;
			
		case 1:
			m_nSpeed += 2;
			break;
			
		default:
			break;
		}
		
		if(m_nSpeed < MIN_RATE)  //最小为1/4倍数
		{
			m_nSpeed = MIN_RATE;
			return false;
		}
		
		if(m_nSpeed > MAX_RATE) //最大为4倍
		{
			m_nSpeed = MAX_RATE;
			return false;
		}
		controlAudio();
		changePlaySpeed(m_nFrameRate);
		
		return true;
	}
	
	/**
	 * 获取当前播放速度
	 * @return 失败返回 -1, 成功返回具体倍数(正数表示快放的倍数，负数表示慢放的倍数)
	 */
	public int getPlaySpeed()
	{
		return m_nSpeed;
	}
	
	// 抓图
	public boolean snapPicture(String strFileName, int nQuality)
	{
		if (null == m_playView)
		{
			return false;
		}
		
//		return m_playView.snapPicture(strFileName, nQuality);
		
		return NativePlayer.snapPicture(m_nativePlayer, strFileName, nQuality);
	}
	
	// 打开音频
	public boolean openAudio()
	{
		if (0 == m_nativePlayer)
		{
			return false;
		}
		m_playSound = true;
		if(m_nSpeed != 0 || m_byFrame)
		{
			return false;
		}
		return NativePlayer.playSound(m_nativePlayer);
	}
	
	// 关闭音量
	public boolean closeAudio()
	{
		boolean close = closeAllAudio();
		
		
		
//		if (null != m_audioRender && m_audioRender.stopSound())
//		{
//			close = true;
//		}
		
		if (!NativePlayer.stopSound(m_nativePlayer))
		{
			close =  false;
		}
		
		m_playSound = false;
		return close;
	}
	
	private boolean closeAllAudio()
	{
		boolean isClose = false;
		
		for (AudioRender audioRender : m_audioRender.values())
		{
			if (null != audioRender && audioRender.stopSound())
			{
				isClose = true;
			}
		}
		
		return isClose;
	}

	/**
	 * 在快、慢放或是按帧的情况下不对声音进行解码播放
	 */
	private void controlAudio()  
	{
		if(m_playSound)
		{
			if(m_nSpeed != 0 || m_byFrame)
			{
				NativePlayer.stopSound(m_nativePlayer);
			}
			else
			{
				NativePlayer.playSound(m_nativePlayer);
			}
		}
		
	}
	
	// 获取音量0~100
	public short getAudioVolume()
	{
		if (null == m_audioRender)
		{
			return -1;
		}
		
		return 0;//m_audioRender.getVolume();
	}
	
	// 设置音量
	public boolean setAudioVolume(short nVolume)
	{
		if (null == m_audioRender)
		{
			return false;
		}
		
		return true;//m_audioRender.setVolume(nVolume);
	}
	
	@Override
	public void onVideoInfo(int nInfoType, int nFrameRate, int nWidth, int nHeight, int nTime)
	{
		final int frameRate = nFrameRate;
		switch (nInfoType)
		{
//		case INFO_TYPE_FRAME_UPDATE:
//			updateFrame(nFrameRate, nWidth, nHeight);
//			break;
			
		case INFO_TYPE_RESOLUTION_CHANGED:
			resolutionChanged(nWidth, nHeight);
			break;
			
		case INFO_TYPE_FRAME_RATE_CHANGED:
			new Thread(new Runnable() {
				
				@Override
				public void run() {
					// TODO Auto-generated method stub
					frameRateChanged(frameRate);
				}
			}).start();
			break;
			
		case INFO_TYPE_TIME_UPDATE:
			break;
			
		case INFO_TYPE_NOT_SUPPORT_VIDEO_FORMAT:
			if (null != m_playListener)
			{
				m_playListener.onNotSupportStream(this);
			}
			break;
		}
	}

	private void frameRateChanged(int nFrameRate)
	{
		try
		{
			if (nFrameRate != m_nFrameRate && nFrameRate != 0)
			{
				m_nFrameRate = nFrameRate;
				
				changePlaySpeed(m_nFrameRate);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	private void resolutionChanged(int nWidth, int nHeight)
	{
		if ((nWidth != m_nWidth) || (nHeight != m_nHeight))
		{
			if (null != m_playListener)
			{
				m_playListener.onSolutionChanged(this, nWidth, nHeight);
			}
			
			m_nWidth  = nWidth;
			m_nHeight = nHeight;
		}
	}

	@Override
	public void onAudioInfo(int nChannels, int nBitsPerSample, int nSamplesPerSecond, int nChannelID)
	{
		System.out.println("nChannels = " + nChannels + ", nBitsPerSample = " + nBitsPerSample + ", nSamplesPerSecond = " + nSamplesPerSecond);
		
		if ((-1 == nChannels) || (-1 == nBitsPerSample) || (-1 == nSamplesPerSecond))
		{
			PlayEvent playEvent = new PlayEvent();
			playEvent.NotSupportAudioFormat = true;
			if(m_playListener != null)
			{
				if(m_audioRender != null)
				{
					//m_audioRender.stopSound();
					closeAllAudio();
				}
				m_playListener.onPlayStatus(playEvent);
			}
			else
			{
				
			}
			return;
		}
		
		if (!m_audioRender.containsKey(nChannelID))
		{
			AudioRender audioRender = new AudioRender();
			if (null == m_audioRender)
			{
				return;
			}
			
			m_audioRender.put(nChannelID, audioRender);
		}
		
		m_audioRender.get(nChannelID).stopSound();
		m_audioRender.get(nChannelID).playSound(nChannels, nBitsPerSample, nSamplesPerSecond);
		
		VoiceIntercom.instance().backAudioParam(nChannels, nBitsPerSample, nSamplesPerSecond,m_audioRender.get(nChannelID));
	}

	@Override
	public void onAudioData(byte[] audioData, int nOffset, int nDataLen, int nChannelID)
	{
//		if(VoiceIntercom.instance().isRecord())
//		{
//			VoiceIntercom.instance().backAudioData(audioData,nOffset,nDataLen);
//		}
//		else
//		{
			m_audioRender.get(nChannelID).inputData(audioData, nOffset, nDataLen);
//		}
	}
	
	@Override
	public void onLostFrame()
	{
		if (null != m_playListener)
		{
			m_playListener.onLostFrame(this);
		}
	}
	
	@Override
	public void onListInfo(int nListSize)
	{
	}
	
	@Override
	public void onPlayPosition(int nCurTime)
	{
		if (null != m_playListener)
		{
			m_playListener.onPlayPosition(this, 0, 0, nCurTime);
		}
	}
	
	@Override
	public void onFrameData(byte[] avData, int nOffset, int nLen)
	{
		//录像
		if((m_record != null) && (m_record.isRecording()))
		{
			if(!m_record.recordData(avData, nOffset, nLen))
			{
				if (null != m_playListener)
				{
					PlayEvent event = new PlayEvent();
					event.SDCardFull = true;
					m_playListener.onPlayStatus(event);
					stopRecord();
				}
			}
		}
	}
	
	@Override
	public void onError(int nFuncType, int nErrCode, String sErrMsg)
	{
		if (nFuncType == FUNC_TYPE_RECORDER)
		{
			Log.e("AVPlayer", "recorder error: " + nErrCode + ", msg: " + sErrMsg);
			if (nErrCode == ERR_RECORDER_WRITE)
			{
				PlayEvent event = new PlayEvent();
				event.SDCardFull = true;
				m_playListener.onPlayStatus(event);
				stopRecord();
			}
		}
	}
	
	
	private void changePlayStatus(PLAY_STATUS ePlayStatus)
	{
		synchronized (m_ePlayStatus)
		{
			m_ePlayStatus = ePlayStatus;
		}
	}
	
	public long getNativePlayer()
	{
		return m_nativePlayer;
	}
	
	synchronized private void changePlaySpeed(int nFrameRate)
	{
		if (null != m_task)
		{
			m_task.cancel();
		}
		
		if (null != m_timer)
		{
			m_timer.cancel();
		}
		
		m_task  = new RenderTimerTask();
		m_timer = new Timer();
		
		float nCurFrameRate = 0;
		if (m_nSpeed < 0)
		{
			nCurFrameRate = (float)nFrameRate / (float)(-m_nSpeed);
			if(nCurFrameRate <=0)
			{
				nCurFrameRate = 1;
			}
		}
		else if (m_nSpeed > 0)
		{
			nCurFrameRate = nFrameRate * m_nSpeed;
		}
		else
		{
			nCurFrameRate = nFrameRate;
		}
		int speed = (int) (1000/nCurFrameRate);
		m_timer.scheduleAtFixedRate(m_task, 0, speed);
	}
	
	// rgbData  解码后RGB数据存放点
	// dataInfo	解码后的信息
	// bDecode	是否真的解码
	public int decodeOneVideoFrame(byte[] rgbData, int[] dataInfo, boolean bDecode)
	{
		if (m_nativePlayer != 0)
		{
			return NativePlayer.decodeOneVideoFrame(m_nativePlayer, rgbData, dataInfo, bDecode);
		}
		
		return -1;
	}
	
	public void enableRender(boolean bRender)
	{
		if (null != m_playView)
		{
			m_playView.enableRender(bRender);
		}
	}
//	private void updateFrame(int nFrameRate, int nWidth, int nHeight)
//	{
//		try
//		{
//			if (nFrameRate != m_nFrameRate && nFrameRate != 0)
//			{
//				m_nFrameRate = nFrameRate;
//				
//				changePlaySpeed(m_nFrameRate);
//			}
//		}
//		catch(Exception e)
//		{
//			e.printStackTrace();
//		}
//		
//		if ((nWidth != m_nWidth) || (nHeight != m_nHeight))
//		{
//			if (null != m_playListener)
//			{
//				m_playListener.onSolutionChanged(this, nWidth, nHeight);
//			}
//			
//			m_nWidth  = nWidth;
//			m_nHeight = nHeight;
//		}
//	}
	
	/**
	 * 开始录像
	 * @param strFilePath
	 * @return
	 */
	public boolean startRecord(String strFilePath) {
		if (m_record == null)
		{
			m_record = new FileRecorder();
		}
		 boolean bRet = m_record.openRecord(strFilePath);
		 if (!bRet) {
			 return false;
		 }
		 int nRet = NativePlayer.startRecord(m_nativePlayer, strFilePath);
		 if (nRet != 0) {
			 return false;
		 }
		 return true;
	}

	/**
	 * 停止录像
	 */
	public void stopRecord() {
		if (m_record == null)
		{
			return;
		}
		NativePlayer.stopRecord(m_nativePlayer);
		m_record.closeRecord();
	}

	public void EnableImageScale(boolean isScale) 
	{
		if (0 != m_nativePlayer)
		{
			NativePlayer.setScaleImage(m_nativePlayer, isScale);
		}
	}
	
	/**
	 * EPTZ相对当前状态进行缩放
	 * @param scale 
	 */
	public void preScale(float scale)
	{
		if (null != m_playView)
		{
			if (m_playView instanceof BasicGLSurfaceView)
			{
				((BasicGLSurfaceView) m_playView).preScale(scale);
			}
			else if (m_playView instanceof BasicSurfaceView) {
				((BasicSurfaceView) m_playView).preScale(scale);
			}
		}
	}
	
	/**
	 * EPTZ绝对缩放
	 * @param scale 
	 */
	public void scale(float scale)
	{
		if (null != m_playView)
		{
			if (m_playView instanceof BasicGLSurfaceView)
			{
				((BasicGLSurfaceView) m_playView).scale(scale);
			}
			else if (m_playView instanceof BasicSurfaceView) {
				((BasicSurfaceView) m_playView).scale(scale);
			}
		}
	}
	
	/**
	 * EPTZ平移
	 * @param x
	 * @param y
	 */
	public void translate( float x, float y)
	{
		if (null != m_playView)
		{
			if (m_playView instanceof BasicGLSurfaceView)
			{
				((BasicGLSurfaceView) m_playView).translate(x, y);
			}
			else if (m_playView instanceof BasicSurfaceView) {
				((BasicSurfaceView) m_playView).translate(x, y);
			}
		}
	}

	/**
	 * 重置EPTZ
	 */
	public void setIdentity()
	{
		if (null != m_playView)
		{
			if (m_playView instanceof BasicGLSurfaceView)
			{
				((BasicGLSurfaceView) m_playView).setIdentity();
			}
			else if (m_playView instanceof BasicSurfaceView) {
				((BasicSurfaceView) m_playView).setIdentity();
			}
		}
	}
	
	/**
	 * 开始EPTZ
	 */
	public void startEPTZ()
	{
		if (null != m_playView)
		{
			if (m_playView instanceof BasicGLSurfaceView)
			{
				((BasicGLSurfaceView) m_playView).startEPTZ();
			}
			else if (m_playView instanceof BasicSurfaceView) {
				((BasicSurfaceView) m_playView).startEPTZ();
			}
		}
	}
	
	/**
	 * 停止EPTZ
	 */
	public void stopEPTZ()
	{
		if (null != m_playView)
		{
			if (m_playView instanceof BasicGLSurfaceView)
			{
				((BasicGLSurfaceView) m_playView).stopEPTZ();
			}
			else if (m_playView instanceof BasicSurfaceView) {
				((BasicSurfaceView) m_playView).stopEPTZ();
			}
		}
	}
	
	/**
	 * 获取EPTZ当前缩放倍数
	 * @return
	 */
	public float getScale() 
	{
		if (null != m_playView)
		{
			if (m_playView instanceof BasicGLSurfaceView)
			{
				return ((BasicGLSurfaceView) m_playView).getScale();
			}
			else if (m_playView instanceof BasicSurfaceView) {
				return ((BasicSurfaceView) m_playView).getScale();
			}
		}
		return -1;
	}
	
	/**
	 * 获取EPTZ当前平移的X值
	 * @return
	 */
	public float getTranslateX() 
	{
		if (null != m_playView)
		{
			if (m_playView instanceof BasicGLSurfaceView)
			{
				return ((BasicGLSurfaceView) m_playView).getTranslateX();
			}
			else if (m_playView instanceof BasicSurfaceView) {
				return ((BasicSurfaceView) m_playView).getTranslateX();
			}
		}
		return -1;
	}
	
	/**
	 * 获取EPTZ当前平移的Y值
	 * @return
	 */
	public float getTranslateY() 
	{
		if (null != m_playView)
		{
			if (m_playView instanceof BasicGLSurfaceView)
			{
				return ((BasicGLSurfaceView) m_playView).getTranslateY();
			}
			else if (m_playView instanceof BasicSurfaceView) {
				return ((BasicSurfaceView) m_playView).getTranslateY();
			}
		}
		return -1;
	}
}
