﻿package com.mm.android.avplaysdk;

/**
 * 实现此接口后可以获得播放过程中的一些事件，如分辨率改变，不支持的码流等
 * 
 *
 */
public interface IPlayListener
{
	/**
	 * 分辨率改变时调用
	 * @param hPlayHandle 产生该事件的播放句柄
	 * @param nWidth 视频宽
	 * @param nHeight 视频高
	 * @return 0处理，-1未处理
	 */
	int onSolutionChanged(IPlayHandle hPlayHandle, int nWidth, int nHeight);
	
	/**
	 * 播放进度，nStartTime：文件开始时间，nEndTime：文件结束时间，nCurTime：当前时间
	 * @param hPlayHandle 产生该事件的播放句柄
	 * @param nStartTime 视频开始时间，在文件播放时有用，回放时录像的开始事件客户端已知，单位s
	 * @param nEndTime 视频结束时间，在文件播放时有用，回放时录像的开始事件客户端已知，单位s
	 * @param nCurTime 当前播放的时间，单位s
	 * @return 0处理，-1未处理
	 */
	int onPlayPosition(IPlayHandle hPlayHandle, int nStartTime, int nEndTime, int nCurTime);
	
	/**
	 * 实时播放丢帧时调用，回放不会主动丢帧
	 * @param hPlayHandle 产生该事件的播放句柄
	 * @return 0处理，-1未处理
	 */
	int onLostFrame(IPlayHandle hPlayHandle);
	
	/**
	 * 出现不支持的码流时调用
	 * @param hPlayHandle 产生该事件的播放句柄
	 * @return 0处理，-1未处理
	 */
	int onNotSupportStream(IPlayHandle hPlayHandle);
	
	
	/**
	 * 播放中的状态回调
	 * @param hPlayHandle
	 * @param playEvent 目前用于回调I帧队列大小，以后可以根据需要继续添加
	 * @return
	 */
	int onPlayStatus(PlayEvent playEvent);
}

