﻿package com.mm.android.avplaysdk.engine;

import java.util.LinkedList;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import com.mm.android.avplaysdk.AVPlayer;
import com.mm.android.avplaysdk.nativeplayer.NativePlayer;


public class EngineModel
{
	public static int SEND_DATA_MIN_THRESHOLD  = 500;
	
	enum MSG_TYPE
	{
		MSG_TYPE_SEND_DATA,
		MSG_TYPE_EXIT
	}
	
	class EngineMessage
	{
		AVPlayer playerFrom = null;
		MSG_TYPE eMsgType	= MSG_TYPE.MSG_TYPE_EXIT;
	}
	
	private static byte[] 	   			   m_engineLock    = new byte[0];
	private static EngineModel 			   m_engineModel   = null;
	private long			   			   m_nativeEngine  = 0;
	private boolean			   			   m_bStop		   = false;
	private Thread			   			   m_engineThread  = null;
	private BlockingQueue<EngineMessage>   m_msgBlockQueue = null;
	private LinkedList<PlayGroup>		   m_listPlayGroup = null;
	private boolean						   m_bInitilize	   = false;
	private	byte[]						   m_initLock	   = new byte[0];
	
	private EngineModel()
	{
		m_msgBlockQueue = new LinkedBlockingQueue<EngineModel.EngineMessage>();
		m_listPlayGroup = new LinkedList<PlayGroup>();
	}
	
	public static EngineModel instance()
	{
		if (null == m_engineModel)
		{
			synchronized (m_engineLock)
			{
				if (null == m_engineModel)
				{
					m_engineModel = new EngineModel();
				}
			}
		}
		
		return m_engineModel;
	}
	
	// start
	public boolean start(long nMinThreshold)
	{
		synchronized (m_initLock)
		{
			if (m_bInitilize)
			{
				return true;
			}
		}
		
		m_nativeEngine = NativePlayer.createEngine(nMinThreshold);
		if (0 == m_nativeEngine)
		{
			return false;
		}
		
		m_bStop = false;
		
		m_engineThread = new Thread (new EngineThread(),"Engine Thread");
		m_engineThread.start();	
		
		synchronized (m_initLock)
		{
			m_bInitilize = true;
		}
		
		return true;
	}
	
	// stop
	public boolean stop()
	{
		if (m_bStop)
		{
			return false;
		}
		
		m_bStop = true;
		
		notifyStop();
		
		try
		{
			m_engineThread.join();
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
		
		boolean bRetValue = NativePlayer.destroyEngine(m_nativeEngine);
		if (!bRetValue)
		{
			return false;
		}
		
		m_nativeEngine = 0;
		
		removeAllPlayGroup();
		
		return true;
	}
	
//	// set player
//	public void setPlayer(long player)
//	{
//		m_nativePlayer = player;
//	}
//	
//	// set indexer
//	public void setIndexer(long indexer)
//	{
//		m_nativeIndexer = indexer;
//	}
	
	public void addPlayGroup(PlayGroup playGroup)
	{
		synchronized (m_listPlayGroup)
		{
			m_listPlayGroup.add(playGroup);
		}
	}
	
	public void removePlayGroup(PlayGroup playGroup)
	{
		synchronized (m_listPlayGroup)
		{
			synchronized (playGroup.getPlayer())
			{
				m_listPlayGroup.remove(playGroup);
			}
			
		}
	}
	
	private void removeAllPlayGroup()
	{
		synchronized (m_listPlayGroup)
		{
			m_listPlayGroup.clear();
		}
	}
	
	// send data
	public void notifySendData(AVPlayer player)
	{
//		查找对应的player
//		synchronized (m_sendDataEvent)
//		{
//			m_sendDataEvent.notify();
//		}
		synchronized (m_msgBlockQueue)
		{
			boolean bAlreadyExist = false;
			EngineMessage msg = m_msgBlockQueue.peek();
			if (msg != null)
			{
				for (EngineMessage engineMsg : m_msgBlockQueue)
				{
					if (engineMsg.playerFrom == player)
					{
						bAlreadyExist = true;
						break;
					}
				}
			}

			if (!bAlreadyExist)
			{
				EngineMessage engineMsg = new EngineMessage();
				engineMsg.playerFrom    = player;
				engineMsg.eMsgType		= MSG_TYPE.MSG_TYPE_SEND_DATA;
				m_msgBlockQueue.add(engineMsg);
			}
		}
	}
	
	private void notifyStop()
	{
		synchronized (m_msgBlockQueue)
		{
			EngineMessage engineMsg = new EngineMessage();
			engineMsg.playerFrom    = null;
			engineMsg.eMsgType		= MSG_TYPE.MSG_TYPE_EXIT;
			m_msgBlockQueue.add(engineMsg);
		}
	}
	
	// engine thread
	class EngineThread implements Runnable
	{

		@Override
		public void run()
		{
			while (!m_bStop)
			{
				EngineMessage engineMsg = null;
				try
				{
					engineMsg = m_msgBlockQueue.take();
				}
				catch (InterruptedException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
				if (engineMsg.eMsgType == MSG_TYPE.MSG_TYPE_EXIT)
				{
					break;
				}
				PlayGroup playGroup = null;
				int nRetValue = 0;
				long indexer = 0;
				synchronized (engineMsg.playerFrom)
				{
					playGroup = getPlayGroup(engineMsg.playerFrom);
					if (null == playGroup)
					{
						continue;
					}
									
					// 从索引中读取数据送入播放库中播放
					indexer = playGroup.getDataSource().getIndexer().getIndexer();
					//Log.d("logic", "EngineThread : sendData");
					
					nRetValue = NativePlayer.sendData(m_nativeEngine, indexer, 
							  playGroup.getPlayer().getNativePlayer(), 
							  SEND_DATA_MIN_THRESHOLD);
				
					if (NativePlayer.SEND_DATA_NO_MORE_DATA == nRetValue)
					{
						playGroup.getDataSource().notifyInputData(NativePlayer.getIndexRemainTime(indexer));
					}
				}
				
				if (NativePlayer.SEND_DATA_LIST_FULL == nRetValue)
				{
					removeEngineMsg(playGroup.getPlayer());
				}
				
//				if (NativePlayer.SEND_DATA_LIST_FULL == nRetValue)
//				{
//					System.out.println("send data failed--->list full");
//					
//					try
//					{
//						synchronized (m_sendDataEvent)
//						{
//							m_sendDataEvent.wait();
//						}
//					}
//					catch (InterruptedException e)
//					{
//						// TODO Auto-generated catch block
//						e.printStackTrace();
//					}
//				}
//				else if (NativePlayer.SEND_DATA_NO_MORE_DATA == nRetValue)
//				{
//					//Object m_receiveDataEvent = new Object();
//					// 通知数据发送模块向索引模块送入数据
//					synchronized (m_receiveDataEvent)
//					{
//						m_receiveDataEvent.notify();
//					}
//					System.out.println("send data failed--->no more data");
//				}
//				else
//				{
//					//
//				}
			}
		}
		
		private PlayGroup getPlayGroup(AVPlayer player)
		{
			synchronized (m_listPlayGroup)
			{
				for (PlayGroup playGroup : m_listPlayGroup)
				{
					if (playGroup.getPlayer() == player)
					{
						return playGroup;
					}
				}
			}
			
			return null;
		}
	}
	
	private void removeEngineMsg(AVPlayer player)
	{
		synchronized (m_msgBlockQueue)
		{
			for (EngineMessage engineMsg : m_msgBlockQueue)
			{
				if (engineMsg.playerFrom == player)
				{
					m_msgBlockQueue.remove(engineMsg);
				}
			}
		}
	}
}
