﻿package com.mm.android.avplaysdk.indexer;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import com.mm.android.avplaysdk.AVErrorCenter;
import com.mm.android.avplaysdk.AVPlaySDK;
import com.mm.android.avplaysdk.IPlayListener;
import com.mm.android.avplaysdk.PlayEvent;
import com.mm.android.avplaysdk.nativeplayer.NativePlayer;


public class FileDataSource extends DataSource implements Runnable
{
	private Thread  		 m_fileThread 	  = null;				// 对文件建索引及送数据的线程
	private boolean 		 m_bStop	   	  = false;
	private String			 m_strFilePath 	  = null;
	private RandomAccessFile m_fileInput   	  = null;
	private PlayFileIndex	 m_indexIFrame 	  = null;				// I帧的索引
	private byte[] 			 m_data  	   	  = new byte[1024*4];
//	private Thread			 m_inputThread 	  = null;				// 送数据到回放索引模块的线程
	private byte[]			 m_inputDataEvent = new byte[1];
	private int				 m_nStartTime	  = 0;
	private int				 m_nEndTime		  = 0;
	private long			 m_nFilePos		  = 0;
	private byte[]			 m_filePosLock	  = new byte[1];
	private IPlayListener 	 m_listener	= null;
	
	public void setPlayListener(IPlayListener listener)
	{
		m_listener = listener;
	}
	
	public FileDataSource(String strFilePath)
	{
		m_strFilePath = strFilePath;
	}
	
	public boolean start()
	{
		m_bStop = false;
		
		try
		{
			m_fileInput = new RandomAccessFile(m_strFilePath, "r");
		}
		catch (FileNotFoundException e)
		{
			AVErrorCenter.put(AVPlaySDK.AVPLAY_FILE_ERROR);
			
			e.printStackTrace();
			return false;
		}
		
		m_indexIFrame = new PlayFileIndex(m_strFilePath);
		
		m_fileThread = new Thread(this);
		m_fileThread.start();
		
//		m_inputThread = new Thread(new InputStreamThread());
		
		return true;
	}
	
	public boolean stop()
	{
		m_bStop = true;
		
		if (null == m_fileThread)
		{
			return false;
		}
		
		try
		{
			synchronized (m_inputDataEvent)
			{
				m_inputDataEvent.notify();
			}
			
			m_fileThread.join();
			//m_inputThread.join();
			
			m_fileInput.close();
		}
		catch (InterruptedException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (IOException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
//		m_indexThread = null;
//		m_inputThread = null;
		m_fileThread  = null;
		
		m_indexIFrame.cleanAll();
		
		return true;
	}
	
	public void destroy()
	{
		m_indexIFrame.destroy();
		super.destroy();
	}
	
	@Override
	public void run()
	{
//		int nReadLen = 0;
		
		createIndex();
		m_nStartTime = m_indexIFrame.getStartTime();
		m_nEndTime	 = m_indexIFrame.getEndTime();
		if(m_listener != null)
		{
			PlayEvent event = new PlayEvent();
			event.IFrameListSize = m_indexIFrame.getIndexCount();
			m_listener.onPlayStatus(event);
		}
		inputStream();
		
		
		
//		while (!m_bStop)
//		{
//			try
//			{
//				nReadLen = m_fileInput.read(m_data);
//				if (nReadLen == -1)
//				{
//					break;
//				}
//				
//				while (!NativePlayer.inputStream(m_indexIFrame.getIndexer(), m_data, 0, nReadLen))
//				{
//					Thread.sleep(10);
//				}
//			} catch (IOException e)
//			{
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			} catch (InterruptedException e)
//			{
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
//		}
		
		
		
//		m_inputThread.start();
	}
	
	private void createIndex()
	{
		int nReadLen = 0;
		
		while (!m_bStop)
		{
			try
			{
				nReadLen = m_fileInput.read(m_data);
				if (nReadLen == -1)
				{
					break;
				}
				
				while (!NativePlayer.inputStream(m_indexIFrame.getIndexer(), m_data, 0, nReadLen))
				{
					Thread.sleep(10);
				}
			} catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	private void inputStream()
	{
		long nFileLength = 0;
		try
		{
			m_fileInput.seek(0);
			nFileLength = m_fileInput.length();
		} catch (IOException e1)
		{
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		while (!m_bStop)
		{
			int nReadLen = 0;
			
			try
			{
				synchronized (m_filePosLock)
				{
					m_fileInput.seek(m_nFilePos);
				}
				
				nReadLen = m_fileInput.read(m_data, 0, m_data.length);
				if (nReadLen <= 0)
				{
					try
					{
						Thread.sleep(5);
					}
					catch (InterruptedException e)
					{
						e.printStackTrace();
					}
					
					continue;
				}
				
				synchronized (m_filePosLock)
				{
					m_nFilePos += nReadLen;
					if (m_nFilePos > nFileLength)
					{
						m_nFilePos = nFileLength;
					}
				}
				
				if (!NativePlayer.inputStream(m_indexPlay.getIndexer(), m_data, 0, nReadLen))
				{
					synchronized (m_inputDataEvent)
					{
						try
						{
							if (!m_bStop)
							{
								m_inputDataEvent.wait();
							}
						} catch (InterruptedException e)
						{
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
				}
			} catch (IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}
	
//		
//		@Override
//		public void run()
//		{
////			try
////			{
////				outputStream = new FileOutputStream("/sdcard/jav_outputstream.dav");
////			} catch (FileNotFoundException e2)
////			{
////				// TODO Auto-generated catch block
////				e2.printStackTrace();
////			}
////			
////			
////			
////			long nFileLength = 0;
////			try
////			{
////				m_fileInput.seek(0);
////				nFileLength = m_fileInput.length();
////			} catch (IOException e1)
////			{
////				// TODO Auto-generated catch block
////				e1.printStackTrace();
////			}
////			
////			IndexInfo startIndexInfo = m_indexIFrame.getOneIndex();
////			IndexInfo endIndexInfo	 = null;
////			long nTotalInputLength	 = 0;
////			
////			while (!m_bStop)
////			{
////				//startIndexInfo = m_indexIFrame.getOneIndex();
////				if (startIndexInfo == null)
////				{
////					break;
////				}
////				
////				endIndexInfo   = m_indexIFrame.getOneIndex();
////				if (endIndexInfo == null)
////				{
////					nTotalInputLength = nFileLength - startIndexInfo.nFilePos;
////				}
////				else
////				{
////					nTotalInputLength = endIndexInfo.nFilePos - startIndexInfo.nFilePos;
////				}
////				
////				try
////				{
////					m_fileInput.seek(startIndexInfo.nFilePos);
////					int nAlreadInputLen = 0;
////					int nReadLen 		= 0;
////					int nLength			= 0;
////					while ((nAlreadInputLen < nTotalInputLength) && !m_bStop)
////					{
////						if ((nTotalInputLength - nAlreadInputLen) < m_data.length)
////						{
////							nLength = (int) (nTotalInputLength - nAlreadInputLen);
////						}
////						else
////						{
////							nLength = m_data.length;
////						}
////						
////						nReadLen = m_fileInput.read(m_data, 0, nLength);
////						
////						
//////						outputStream.write(m_data, 0, nReadLen);
////						
////						
////						if (!NativePlayer.inputStream(m_indexPlay.getIndexer(), m_data, 0, nReadLen))
////						{
////							synchronized (m_inputDataEvent)
////							{
////								try
////								{
////									m_inputDataEvent.wait();
////								} catch (InterruptedException e)
////								{
////									// TODO Auto-generated catch block
////									e.printStackTrace();
////								}
////							}
////						}
////						nAlreadInputLen += nReadLen;
////					}
////					
////				} catch (IOException e)
////				{
////					// TODO Auto-generated catch block
////					e.printStackTrace();
////				}
////				
////				startIndexInfo = endIndexInfo;
////			}
////			
////			try
////			{
////				outputStream.close();
////			} catch (IOException e)
////			{
////				// TODO Auto-generated catch block
////				e.printStackTrace();
////			}
//		}
//	}

	@Override
	public void notifyInputData(int nRemainTime)
	{
		synchronized (m_inputDataEvent)
		{
			m_inputDataEvent.notify();
		}
	}
	
	public int getStartTime()
	{
		return m_nStartTime;
	}
	
	public int getEndTime()
	{
		return m_nEndTime;
	}
	
	public boolean setPlayPos(int nPlayTime)
	{
		if (!m_indexIFrame.setPlayPos(nPlayTime))
		{
			return false;
		}
		
		IndexInfo indexInfo = m_indexIFrame.getOneIndex();
		if (indexInfo == null)
		{
			return false;
		}
		
		synchronized (m_filePosLock)
		{
			m_nFilePos = indexInfo.nFilePos;
		}
		
		return true;
	}
}
