﻿package com.mm.android.avplaysdk.indexer;

import java.util.ArrayList;

import com.mm.android.avplaysdk.nativeplayer.NativePlayer;


public class PlayFileIndex extends Indexer implements IIndexListener
{
	private ArrayList<IndexInfo> m_listIndexInfo = null;
	private int					 m_nIndexPos	 = 0;
	private byte[]				 m_indexPosLock	 = new byte[0];
	private int					 m_nEndTime		 = 0;
	
	public PlayFileIndex(String strFilePath)
	{
		super(strFilePath);
		m_listIndexInfo = new ArrayList<IndexInfo>();
		
		NativePlayer.setIndexListener(m_indexer, this);
	}

	@Override
	public boolean onAddOneIndex(IndexInfo indexInfo)
	{
		m_nEndTime = indexInfo.nDateTime;
		
		if (indexInfo.nFilePos >= 0)
		{
			return m_listIndexInfo.add(indexInfo);
		}
		
		return true;
	}
	
	public IndexInfo getOneIndex()
	{
		IndexInfo indexInfo = null;
		
		synchronized (m_indexPosLock)
		{
			if (m_nIndexPos < m_listIndexInfo.size())
			{
				indexInfo = m_listIndexInfo.get(m_nIndexPos);
			}
		
			m_nIndexPos++;
		}
		
		return indexInfo;
	}
	
	public int getIndexCount()
	{
		return m_listIndexInfo.size();
	}
	
	public void cleanAll()
	{
		m_listIndexInfo.clear();
	}
	
	public int getStartTime()
	{
		if (m_listIndexInfo.size() == 0)
		{
			return 0;
		}
		
		return m_listIndexInfo.get(0).nDateTime;
	}
	
	public int getEndTime()
	{
		return m_nEndTime;
//		int nListSize = m_listIndexInfo.size();
//		if (nListSize == 0)
//		{
//			return 0;
//		}
//		
//		return m_listIndexInfo.get((nListSize - 1)).nDateTime;
	}
	
	public boolean setPlayPos(int nPlayTime)
	{
		int nListSize = m_listIndexInfo.size();
		
		if (nPlayTime <= getStartTime())
		{
			synchronized (m_indexPosLock)
			{
				m_nIndexPos = 0;
			}
			return true;
		}
		
		if (nPlayTime >= getEndTime())
		{
			synchronized (m_indexPosLock)
			{
				m_nIndexPos = nListSize - 1;
			}
			return true;
		}
		
		IndexInfo firstIndex  = null;
		IndexInfo secondIndex = null;
		for (int i = 0; i < nListSize - 1; i++)
		{
			firstIndex  = m_listIndexInfo.get(i);
			secondIndex = m_listIndexInfo.get(i+1);
			if ((nPlayTime >= firstIndex.nDateTime) && (nPlayTime <= secondIndex.nDateTime))
			{
				synchronized (m_indexPosLock)
				{
					m_nIndexPos = i;
				}
				break;
			}
		}
		
		return true;
	}
}
