﻿package com.mm.android.avplaysdk.nativeplayer;

import com.mm.android.avplaysdk.indexer.IIndexListener;
import com.mm.android.avplaysdk.param.AVFrameInfo;
import com.mm.android.avplaysdk.render.IRenderListener;


public class NativePlayer
{
	static
	{
		System.loadLibrary("SF_PlaySDK");
	}
	
	public final static int PARSE_MODE_FILE 	   = 1;
	public final static int PARSE_MODE_STREAM      = 0;
	
	// 建立索引的类型
	public final static int INDEX_TYPE_PLAYBACK    = 0;
	public final static int INDEX_TYPE_PLAYFILE    = 1;
	
	// 发送数据的错误返回值
	public final static int SEND_DATA_NO_ERROR	   = 0;		// 没有错误
	public final static int SEND_DATA_NO_MORE_DATA = 1;		// 索引记录中没有数据
	public final static int SEND_DATA_LIST_FULL	   = 2;		// 播放队列已经满了
	
	// 初始化返回码
	public final static int INIT_ERR_OK 	  = 0;
	public final static int INIT_ERR_DECODER  = -1;
	public final static int INIT_ERR_PLAYBACK = -2;
	
	//
	// 函数: startup
	// 描述: 初始化播放库
	// 参数: 无
	// 返回: 成功返回0,失败返回<0
	//
	public static native int startup(String appPackageName,String strPlayBackDir);

	//public static native boolean startup();
	//
	// 函数: cleanup
	// 描述: 反初始化库
	// 参数: 无
	// 返回: 成功返回true,失败返回false.
	//
	public static native boolean cleanup();

	//
	// 函数: createPlayer
	// 描述: 创建播放句柄
	// 参数: mode, 播放模式;nDelayTime, 允许的最大延时时间; bPureAudio, 标记是否是纯音频
	// 返回: 失败返回0,其它表示成功
	//
	public static native long createPlayer(int mode, int nDelayTime, boolean bPureAudio);

	//
	// 函数: destroyPlayer
	// 描述: 销毁播放句柄
	// 参数: player,播放句柄.
	// 返回: 成功返回true,失败返回false.
	//
	public static native boolean destroyPlayer(long player);

	//
	// 函数: play
	// 描述: 开始播放
	// 参数: player,播放句柄.
	//		 wnd,播放窗口
	// 返回: 成功返回true,失败返回false.
	//
	public static native boolean play(long player, long wnd);

	//
	// 函数: inputData
	// 描述: 向播放句柄送数据.
	// 参数: player,播放句柄.
	//		 data,WWAV码流数据.
	//		 nDataLen,码流长度.
	// 返回: 成功返回true,失败返回false.
	//
	public static native int 	 inputData(long player, byte[] data, int nOffset, int nDataLen, boolean bPlay);
	
	//public static native int	 inputDataEx(long player, byte[] data, int nOffset, int nDataLen);

	//
	// 函数: inputFrame
	// 描述: 向播放句柄送数据.
	// 参数: player,播放句柄.
	//		 frameInfo,WWAV码流数据句柄.
	// 返回: 成功返回true,失败返回false.
	//
	public static native int 	 inputFrame(long player, AVFrameInfo frameInfo, byte[] data);
	
	//
	// 函数: stop
	// 描述: 停止播放
	// 参数: player,播放句柄.
	// 返回: 成功返回true,失败返回false.
	//
	public static native boolean stop(long player);
	
	//
	// 函数: setRenderListener
	// 描述: 设置显示监听类，在需要播放库需要操作显示
	//		  模块时会调用IRenderListener中对应的方法.
	// 参数: player,播放句柄.
	//		 renderListener,显示监听.
	// 返回: 成功返回true,失败返回false.
	//
	public static native boolean setRenderListener(long player, IRenderListener renderListener);
	
	//
	// 函数: renderOneFrame
	// 描述: 显示一帧数据，当该显示一帧时调用该接口
	// 参数: player,播放句柄.
	//		 bRender,是否显示，置为true表示显示，false表示不显示，
	//				   主要是由于在横屏和竖屏之间切换时用于显示的View会被销毁
	//		 		   因此需要将应该显示的帧丢弃
	// 返回: 成功返回true,失败返回false.
	//
	public static native boolean renderOneFrame(long player, boolean bRender);
	
	//
	// 函数: surfaceCreated
	// 描述: 当opengl的surface创建时调用，native会对显示模块做一些初始化工作
	// 参数: player,播放句柄.
	// 返回: 
	//
	public static native void surfaceCreated(long player);
	
	//
	// 函数: surfaceChanged
	// 描述: 当opengl的surface宽高发生变化时调用
	// 参数: player,播放句柄.
	//		 nViewWidth,显示窗口的宽
	//		 nViewHeight,显示窗口的高
	// 返回: 
	//
	public static native void surfaceChanged(long player, int nViewWidth, int nViewHeight);
	
	//
	// 函数: playSound
	// 描述: 打开声音
	// 参数: player,播放句柄.
	// 返回: 
	//
	public static native boolean playSound(long player);
	
	//
	// 函数: stopSound
	// 描述: 关闭声音
	// 参数: player,播放句柄.
	// 返回: 
	//
	public static native boolean stopSound(long player);
	
	
	public static native int startRecord(long player, String filepath);

	public static native void  stopRecord(long player);
	
	
	
	//
	// 函数: decodeOneVideoFrame
	// 描述: 解码一帧视频数据
	// 参数: player,播放句柄.
	//		 rgbData,解码后的rgb数据
	//		 dataInfo,存储这一视频帧信息的数据，第一个元素表示视频宽度，第二个元素表示视频高度。
	//				     这样做是为了不在native层调用java层的接口，所以将解码完成后的数据，
	//				     以及视频信息都存放在由java层传入的数组中
	//		 bDecode,是否解码
	// 返回: 此帧rgb数据的长度，如果为零，则表示未对rgbData，dataInfo进行赋值操作，
	//		 上层在显示的时候 需要将上一帧数据再显示一次，防止画面闪烁
	//
	public static native int decodeOneVideoFrame(long player, byte[] rgbData, int[] dataInfo, boolean bDecode);

	//------------------------------------------------------------------------
	// 函数: getDecodeInfo
	// 描述: 获取解码后的相关信息
	// 参数: hPlayer,播放句柄.
	//		lpInfoBuf,存储解码后的信息，lpInfoBuf[0]-宽，lpInfoBuf[1]-高
	//		nInfoBufLen,lpInfoBuf长度
	// 返回: 成功返回true，失败返回false
	//------------------------------------------------------------------------
	public static native boolean getDecodeInfo(long player, int[] fileInfo);
	
	//
	// 函数: getFileInfo
	// 描述: 获取码流文件的相关信息，如码流文件中的开始播放时间和结束播放时间
	// 参数: player,播放句柄.
	//		 fileName,文件全路径
	//		 fileInfo,存储指定文件的相关信息，第一个元素表示文件开始时间，第二个元素表示
	//				     文件结束时间
	// 返回: 成功返回true，失败返回false
	//
	public static native boolean getFileInfo(long player, String fileName, int[] fileInfo);
	
	//
	// 函数: getFrameListSize
	// 描述: 获取播放队列的大小
	// 参数: player,播放句柄.
	//		 
	// 返回: 队列大小，如果返回-1表示该player不存在，返回0表示队列已放完
	//
	public static native int getFrameListSize(long player);
	
	
	public static native int resetPlayer(long player);
	
	
	
	
	
	
	
	
	
	
	/////////////////////////////////////////////////////////////////////////////////
	
	//
	// 函数：createIndexBuilder
	// 描述：创建一个索引建立器对象，用于对回放的数据建立索引
	// 参数： nIndexType,索引建立的类型，INDEX_TYPE_PLAYBACK表示网络回放，INDEX_TYPE_PLAYFILE表示本地文件的播放
	//	     strFilePath,如果是本地文件播放，传入文件路径，否则传入null
	// 返回：索引建立器对象句柄，0表示失败
	//
	public static native long createIndexBuilder(int nIndexType, String strFilePath);
	
	//
	// 函数：destroyIndexBuilder
	// 描述：销毁索引建立器对象
	// 参数：indexer,createIndexBuilder返回的句柄
	// 返回：成功返回true,失败返回false.
	// 
	public static native boolean destroyIndexBuilder(long indexer);
	
	//
	// 函数：createEngine
	// 描述：创建驱动模块，用来根据索引读取相应的数据送入播放库，以及同步控制
	// 参数：nMinThreShold,指定索引中最小的数据量，如果小于这个值，就应该通知网络模块向索引模块送数据
	// 返回：驱动模块句柄，0表示失败
	//
	public static native long createEngine(long nMinThreshold);
	
	//
	// 函数：destroyEngine
	// 描述：销毁驱动模块
	// 参数：createEngine返回的句柄
	// 返回：成功返回true,失败返回false.
	//
	public static native boolean destroyEngine(long engine);
	
	//
	// 函数：inputStream
	// 描述：送数据到索引建立模块中
	// 参数: indexer,索引句柄.
	//		 data,码流数据.
	//		 nOffset,数据在data中的起始位置
	//		 nDataLen,码流长度.
	// 返回: 成功返回true,失败返回false.
	//
	public static native boolean inputStream(long indexer, byte[] data, int nOffset, int nDataLen);
	
	//
	// 函数：sendData
	// 描述：根据索引读取相应的数据送入播放库
	// 参数： engine,createEngine返回的句柄
	//		 indexer,createIndexBuilder返回的句柄
	//		 player,createPlayer返回的句柄
	// 		 timeSpace,要送入播放库中的数据量，以时间为单位
	// 返回：
	//
	public static native int sendData(long engine, long indexer, long player, long timeSpace);
	
	//
	// 函数：setIndexListener
	// 描述：设置索引信息的监听器
	// 参数： indexer,createIndexBuilder返回的句柄
	// 		 indexListener,索引信息监听器
	// 返回：
	//
	public static native boolean setIndexListener(long indexer, IIndexListener indexListener);
	
	//
	// 函数：setIndexListener
	// 描述：设置索引信息的监听器
	// 参数： player,createPlayer返回的句柄
	// 返回：
	//
	public static native boolean clearPlayList(long player);
	
	//
	// 函数：setIndexListener
	// 描述：设置索引信息的监听器
	// 参数： indexer,createIndexBuilder返回的句柄
	// 返回：
	//
	public static native boolean clearIndexList(long indexer);
	
	//
	// 函数：getIndexRemainTime
	// 描述：获取索引队列中剩余的时间
	// 参数： indexer,createIndexBuilder返回的句柄
	// 返回：索引队列中剩余的时间
	//
	public static native int getIndexRemainTime(long indexer);
	
	//
	// 函数：setScaleImage
	// 描述：设置是否对图像进行缩放
	// 参数： player,createPlayer返回的句柄
	//		 isScale,是否对分辨率大于等于D1的图像进行缩放
	// 返回：无
	//
	public static native void setScaleImage(long player, boolean isScale);
	
	//
	// 函数：snapPicture
	// 描述：抓图
	// 参数： player,createPlayer返回的句柄
	//		 filePath,图片保存路径
	//		 quality,图片质量
	// 返回：成功返回ture，失败返回false
	//
	public static native boolean snapPicture(long player, String filePath, int quality);
	
	//
	// 函数：echoInit
	// 描述：回音消除模块初始化
	// 参数： 
	// 返回：成功返回模块句柄，失败返回-1
	//
	public static native long echoInit(int ref_delay_size);
	
	
	public static native byte[] echoGetSignalBuffer();
	
	
	/**
	 * 回音消除
	 * @param handle	回音消除句柄
	 * @param srcData	采集的声音
	 * @param refData	参考声音(播放出来的声音)
	 * @param outData	回音消除后的声音
	 * @param nBitsPerSample	采集声音 采样深度
	 * @param nSamplesPerSecond 采集声音 采样率
	 * @param nBitsPerSampleRef	 参考声音 采样深度
	 * @param nSamplesPerSecondRef  参考声音 采样率
	 * @return
	 */
	public static native int echoCancel(long handle, byte[] srcData, byte[] refData, byte[] outData, int nBitsPerSample,int nSamplesPerSecond,int nBitsPerSampleRef, int nSamplesPerSecondRef);
	
	//
	// 函数：echoDeInit
	// 描述：回音消除模块销毁
	// 参数： 
	// 返回：成功返回0，失败返回-1
	//
	public static native int echoDeInit(long handle);
}

















