﻿package com.mm.android.avplaysdk.render;

import java.util.ArrayList;
import java.util.List;

import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.util.Log;

public class AudioRender implements Runnable
{
	private static final int MAX_SMOOTH_BUF_LEN   = 5*1024;
	private static final int MAX_SMOOTH_AUDIO_LEN = 4*1024;
	
	private int 	   		m_nBufSize;
	private AudioTrack 		m_audioTrack;
	private List<byte[]> 	m_lstAudioData;
	private Thread			m_audioPlayThread;
	private	boolean			m_bStop = false;

	private byte[]			m_smoothBuf    = new byte[MAX_SMOOTH_BUF_LEN * 2];
	private int				m_nCurAudioLen = 0;

	//private FileOutputStream 		m_output 				= null;		
	public AudioRender()
	{
		m_nBufSize 	  = 0;
		m_audioTrack  = null;
		m_lstAudioData = new ArrayList<byte[]>();
		
	}
	
	public void clearBuffer()
	{
		synchronized (m_lstAudioData) {
			m_lstAudioData.clear();
		}
	}
	
	@SuppressWarnings("deprecation")
	public boolean playSound(int nChannels, int nBitsPerSample, int nSamplesPerSecond)
	{
		int channel 		 = AudioFormat.CHANNEL_CONFIGURATION_MONO;
		int bitsPerSample 	 = AudioFormat.ENCODING_PCM_16BIT;
		int samplesPerSecond = 8000;
		
		// 声道数
		if (nChannels == 1)
		{
			channel = AudioFormat.CHANNEL_CONFIGURATION_MONO;
		}
		else if (nChannels == 2)
		{
			channel = AudioFormat.CHANNEL_CONFIGURATION_STEREO;
		}
		else
		{
			//如果码流数据中没有声道数，则采用手机设备默认声道   modify by zhujb
			nChannels = AudioFormat.CHANNEL_CONFIGURATION_DEFAULT;
			//return false;
		}
		
		// 采样深度
		if (nBitsPerSample == 8)
		{
			bitsPerSample = AudioFormat.ENCODING_PCM_8BIT;
		}
		else if (nBitsPerSample == 16)
		{
			bitsPerSample = AudioFormat.ENCODING_PCM_16BIT;
		}
		else
		{
			return false;
		}
		
		// 采样率
		samplesPerSecond = nSamplesPerSecond;
		stopSound();
		
		m_nBufSize   = AudioTrack.getMinBufferSize(samplesPerSecond, channel, bitsPerSample);
		m_audioTrack = new AudioTrack(AudioManager.STREAM_MUSIC, samplesPerSecond,  
									  channel, bitsPerSample, m_nBufSize,  
					                  AudioTrack.MODE_STREAM);
		try
		{
			m_audioTrack.play();
			m_bStop = false;
			
			m_lstAudioData.clear();
			m_nCurAudioLen = 0;
			
			if(m_audioPlayThread == null){
					m_audioPlayThread = new Thread(this);
			}
			m_audioPlayThread.start();
		}
		catch(Exception e)
		{
			
			e.printStackTrace();
			return false;
		}
		
		
		return true;
	}
	
	@SuppressWarnings("deprecation")
	public boolean stopSound()
	{	
//		try
//		{
//			if(m_output != null)
//			{
//				m_output.close();
//			}
//			
//		} catch (IOException e)
//		{
//			e.printStackTrace();
//		}
		
		if (null == m_audioTrack)
		{
			return false;
		}
		
		if(m_audioPlayThread != null){
			try {
				m_bStop = true;
				m_audioPlayThread.join(4000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		if(m_audioPlayThread != null && m_audioPlayThread.isAlive())
		{
			m_audioPlayThread.stop();
			Log.d("stop", "m_audioPlayThread join failed so stop:");
		}
		m_audioPlayThread = null;
		
		try
		{
			m_audioTrack.stop();
			m_audioTrack.release();
			m_audioTrack  = null;
			
			m_lstAudioData.clear();
			m_nCurAudioLen = 0;
			if(m_smoothBuf != null)
			{
				m_smoothBuf = null;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			m_bStop = true;
			m_audioTrack  = null;
			m_lstAudioData.clear();
			return false;
		}
		System.gc();
		
		return true;
	}
	
	public int inputData(byte[] audioData, int nOffset, int nDataLen)
	{
		if (null == m_audioTrack)
		{
			return -1;
		}
		
//		if(m_output == null)
//		{
//			try
//			{
//				String strFileName = "/sdcard/audio.dat";
//				m_output = new FileOutputStream(strFileName);
//			} catch (FileNotFoundException e)
//			{
//				e.printStackTrace();
//			}
//		}
//		
//
//		try
//		{
//			m_output.write(audioData, nOffset, nDataLen);
//		} catch (IOException e)
//		{
//			e.printStackTrace();
//		}

		if(m_smoothBuf == null)
		{
			m_smoothBuf    = new byte[MAX_SMOOTH_BUF_LEN * 2];
		}
		System.arraycopy(audioData, nOffset, m_smoothBuf, m_nCurAudioLen, nDataLen);
		m_nCurAudioLen += nDataLen;
		if (m_nCurAudioLen < MAX_SMOOTH_AUDIO_LEN)
		{
			return 0;
		}

		byte[] byAudioData = new byte[m_nCurAudioLen];
		System.arraycopy(m_smoothBuf, 0, byAudioData, 0, m_nCurAudioLen);
		synchronized(m_lstAudioData){
			if(m_lstAudioData.size() > 5)  //实时监视的时候，同步由播放SDK控制，语音对讲的时候，设置队列的最大长度为5
			{
				m_lstAudioData.clear();
			}
			m_lstAudioData.add(byAudioData);
			m_nCurAudioLen = 0;
		}
		return 0;
	}
	
	public short getVolume()
	{
		return 0;
	}
	
	public boolean setVolume(short nVolume)
	{
		return true;
	}

	/**
	 * 音频播放线程，循环的播放网络接收到，并解码后的数据
	 */
	@Override
	public void run() {
		List<byte[]> audioList = new ArrayList<byte[]>();	// 临时播放队列
		byte[] audioData = null;							// 单块音频数据
		
		while(true){
			
			if(m_bStop){
				break;
			}
			
			//将音频队列拷贝到临时队列中,并清空音频队列
			boolean bNoData = false;
			synchronized(m_lstAudioData){
				if(m_lstAudioData.size() == 0){
					bNoData = true;
				}
				audioList.clear();
				for(byte[] data : m_lstAudioData){ 
					audioList.add(data);
				}
				m_lstAudioData.clear();
			}
			
			// 没有数据的时候，休息一会
			if(bNoData){
				try {
					Thread.sleep(5);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			
			while(true){
				
				if(m_bStop){
					break;
				}
				if(audioList.size() == 0){
					break;
				}
				audioData = audioList.get(0);
				audioList.remove(0);
				if(m_bStop){
					break;
				}
				int nRet = this.m_audioTrack.write(audioData, 0, audioData.length);
				if(nRet != 640 || nRet == AudioTrack.ERROR_INVALID_OPERATION || nRet == AudioTrack.ERROR_BAD_VALUE)
				{
					//Log.d("aa", "AudioTrack write failed! ret: " + nRet);
				}
			}
			
			audioData = null;
		}
		
	}
}
