﻿package com.mm.android.avplaysdk.render;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import com.mm.android.avplaysdk.AVPlaySDK;
import com.mm.android.avplaysdk.AVPlayer;
import com.mm.android.avplaysdk.IViewListener;
import com.mm.android.avplaysdk.nativeplayer.NativePlayer;

import android.opengl.GLSurfaceView;
import android.util.Log;
import android.view.SurfaceView;

public class NativeGLRenderer implements GLSurfaceView.Renderer
{
	private AVPlayer mVideoPlayer = null;
	private boolean m_cleanScreen = false; // 是否需要清屏

	private boolean m_bStop = false; // 是否停止播放，该标志位由其他的线程来改变。
	private byte[] m_StopEvent = new byte[0]; // 已经停止播放的事件，当绘图线程发现m_bStop=true的时候，
											  // 设置m_StopEvent，这样外面的线程就知道已经停止绘图了，
											  // 可以清理资源了【关闭播放等】。

	private int m_nVideoWidth = 0;
	private int m_nVideoHeight = 0;


	private int mRgb = GL10.GL_RGBA;

	int[] videoInfo = new int[3];

	private IViewListener m_viewListener = null;
	private int 		  m_nErrCode 	 = AVPlaySDK.AVPLAY_ERROR;
	private boolean 	  m_bRender 	 = true;
	private SurfaceView   m_surfaceView  = null;
	private long          m_nativeGLESHelp = 0;
	public native long nativeNewGLESHelpObj();
	public native void nativeInit(long gleshelp, int width, int height);
	public native void nativeDraw(long gleshelp, long player, int width, int height, boolean isDraw);
	public native void nativeCleanScreen(long gleshelp);
	public native void nativeDeleteGLESHelpObj(long gleshelp);
	public native void nativeScale(long gleshelpobj, float scale);
	public native void nativePreScale(long gleshelpobj, float scale);
	public native void nativeTranslate(long gleshelpobj, float x, float y);
	public native void nativeSetIdentity(long gleshelpobj);
	public native void nativeStartEPTZ(long gleshelpobj);
	public native void nativeStopEPTZ(long gleshelpobj);
	public native void nativeCleanYUV(long gleshelpobj);
	public native float nativeGetScale(long gleshelpobj);
	public native float nativeGetTranslateX(long gleshelpobj);
	public native float nativeGetTranslateY(long gleshelpobj);
	
	public NativeGLRenderer(SurfaceView glSurfaceView)
	{
		m_nVideoWidth 	= 352;
		m_nVideoHeight 	= 288;
		
		m_surfaceView = glSurfaceView;
		
		m_nativeGLESHelp = nativeNewGLESHelpObj();
	}

	@Override
	public void onDrawFrame(GL10 gl)
	{
		synchronized (this)
		{
			// 清屏
			if (m_cleanScreen)
			{
				nativeCleanScreen(m_nativeGLESHelp);
				nativeCleanYUV(m_nativeGLESHelp);
				m_cleanScreen = false;
			}
			
			if(m_bStop)
			{
			    return;
			}
			
			if(mVideoPlayer != null && m_bRender)
			{
				if(m_nativeGLESHelp == 0) return;
				if(mVideoPlayer.getNativePlayer() == 0) return;
				nativeDraw(m_nativeGLESHelp, mVideoPlayer.getNativePlayer(), m_nVideoWidth, m_nVideoHeight, true);
				int[] decodeInfo = new int[2];
				boolean ret = NativePlayer.getDecodeInfo(mVideoPlayer.getNativePlayer(), decodeInfo);
				if (ret)
				{
					if (decodeInfo[0] != m_nVideoWidth || decodeInfo[1] != m_nVideoHeight)
					{
						mVideoPlayer.onVideoInfo(IRenderListener.INFO_TYPE_RESOLUTION_CHANGED, 0, decodeInfo[0], decodeInfo[1], 0);
						m_nVideoWidth = decodeInfo[0];
						m_nVideoHeight = decodeInfo[1];
					}
				}
			}
		}
	}

	@Override
	public void onSurfaceChanged(GL10 gl, int width, int height)
	{
		synchronized (this)
		{
			if(m_nativeGLESHelp == 0) return;
			nativeInit(m_nativeGLESHelp, width, height);
			
			if(mVideoPlayer != null && !m_bStop)
			{
				if(m_nativeGLESHelp == 0) return;
				if(mVideoPlayer.getNativePlayer() == 0) return;
				nativeDraw(m_nativeGLESHelp, mVideoPlayer.getNativePlayer(), m_nVideoWidth, m_nVideoHeight, true);
				int[] decodeInfo = new int[2];
				boolean ret = NativePlayer.getDecodeInfo(mVideoPlayer.getNativePlayer(), decodeInfo);
				if (ret)
				{
					if (decodeInfo[0] != m_nVideoWidth || decodeInfo[1] != m_nVideoHeight)
					{
						mVideoPlayer.onVideoInfo(IRenderListener.INFO_TYPE_RESOLUTION_CHANGED, 0, decodeInfo[0], decodeInfo[1], 0);
						m_nVideoWidth = decodeInfo[0];
						m_nVideoHeight = decodeInfo[1];
					}
				}
			}
		}
	}

	@Override
	public void onSurfaceCreated(GL10 gl, EGLConfig config)
	{

	}

	public void setPlayer(AVPlayer videoPlayer)
	{
		mVideoPlayer = videoPlayer;
	}

	public void clear()
	{
		m_cleanScreen 	  = true;
	}

	/**
	 * 请求停止绘图
	 */
	public void stopRender()
	{
		synchronized (this)
		{
			m_bStop = true;
			clear();
		}
	}

	/**
	 * 请求开始绘图
	 */
	public void startRender()
	{
		m_cleanScreen = true;
		m_bStop = false;
	}

	public void setViewListener(IViewListener viewListener)
	{
		m_viewListener = viewListener;
	}


	public void enableRender(boolean bRender)
	{
		m_bRender = bRender;
	}
	
	
	public void onlyDecode()
	{
	    synchronized (this)
	    {
	    	if (m_bStop)
	    	{
	    		return;
	    	}
		    nativeDraw(m_nativeGLESHelp, mVideoPlayer.getNativePlayer(), 0, 0, false);
	    }
	}
	
	private void scale()
	{
		if(videoInfo != null && (videoInfo[0] > 1024 || videoInfo[1] > 1024))
		{
			mVideoPlayer.EnableImageScale(true);
		}
		
	}
	
	public void deleteGLESHelp()
	{
		synchronized (this)
		{
			if(m_nativeGLESHelp != 0)
			{
				nativeDeleteGLESHelpObj(m_nativeGLESHelp);
				m_nativeGLESHelp = 0;
			}
		}
	}
	
	public void scale(float scale)
	{
		if(m_nativeGLESHelp != 0)
		{
			nativeScale(m_nativeGLESHelp, scale);
		}
	}
	
	public void preScale(float scale)
	{
		if(m_nativeGLESHelp != 0)
		{
			nativePreScale(m_nativeGLESHelp, scale);
		}
	}
	
	public void translate( float x, float y)
	{
		if(m_nativeGLESHelp != 0)
		{
			nativeTranslate(m_nativeGLESHelp, x, y);
		}
	}

	public void setIdentity()
	{
		if(m_nativeGLESHelp != 0)
		{
			nativeSetIdentity(m_nativeGLESHelp);
		}
	}
	
	public void startEPTZ()
	{
		if(m_nativeGLESHelp != 0)
		{
			nativeStartEPTZ(m_nativeGLESHelp);
		}
	}
	public void stopEPTZ()
	{
		if(m_nativeGLESHelp != 0)
		{
			nativeStopEPTZ(m_nativeGLESHelp);
		}
	}
	public float getScale()
	{
		if(m_nativeGLESHelp != 0)
		{
			return nativeGetScale(m_nativeGLESHelp);
		}
		return -1;
	}
	
	public float getTranslateX()
	{
		if(m_nativeGLESHelp != 0)
		{
			return nativeGetTranslateX(m_nativeGLESHelp);
		}
		return -1;
	}	
	
	public float getTranslateY()
	{
		if(m_nativeGLESHelp != 0)
		{
			return nativeGetTranslateY(m_nativeGLESHelp);
		}
		return -1;
	}	
}
